==> hexo-s-c-dup1-1.0.0.tgz: package/index.js <==
// > 20200523
// This file is modified again by 5945
// changed categories chart to only depth 1 category
// changed categories radar to self defined tag radar

// This file is modified by george16886
// * Translated into English.
// * Remove title of category statistics chart.
// * Enlarge size of category statistics chart.
// * Add post canlendar.
// * Add category radar chart.

//color define
const color1 = '#FDEE34'; //neo yello
const color2 = '#00D7F1'; //neo blue
const color3 = '#3ecf8e'; //green
const color4 = '#fa755a'; //red

const cheerio = require('cheerio')
const moment = require('moment')

hexo.extend.filter.register('after_render:html', function (locals) {
  const $ = cheerio.load(locals)
  const post = $('#posts-chart')
  const tag = $('#tags-chart')
  const category = $('#categories-chart')
  const calendar = $('#posts-calendar')
  const radar = $('#categories-radar')
  let htmlEncode = false

  if (post.length > 0 || tag.length > 0 || category.length > 0 || calendar.length > 0 || radar.length > 0) {
    $('head').after('<style type="text/css">#posts-chart,#posts-calendar,#categories-chart,#categories-radar,#tags-chart{width: 100%;height: 300px;margin: 0.5rem auto;padding: 0.5rem;}</style><script type="text/javascript" src="https://cdn.jsdelivr.net/npm/echarts@4.7.0/dist/echarts.min.js"></script>')
    if (post.length > 0) {
      if (post.attr('data-encode') === 'true') htmlEncode = true
      post.after(postsChart())
    }
    if (calendar.length > 0) {
      if (calendar.attr('data-encode') === 'true') htmlEncode = true
      calendar.after(postsCalendar())
    }
    if (tag.length > 0) {
      if (tag.attr('data-encode') === 'true') htmlEncode = true
      tag.after(tagsChart(tag.attr('data-length')))
    }
    if (category.length > 0) {
      if (category.attr('data-encode') === 'true') htmlEncode = true
      category.after(categoriesChart())
    }
    if (radar.length > 0) {
      if (radar.attr('data-encode') === 'true') htmlEncode = true
      radar.after(categoriesRadar(radar.attr('predef-names')))
    }
    if (htmlEncode) {
      return $.root().html().replace(/&amp;#/g, '&#')
    } else {
      return $.root().html()
    }
  } else {
    return locals
  }
}, 25)

function postsChart () {
  const startDate = moment().subtract(2, 'years').startOf('month')
  const endDate = moment().endOf('month')

  const monthMap = new Map()
  const dayTime = 3600 * 24 * 1000
  for (let time = startDate; time <= endDate; time += dayTime) {
    const month = moment(time).format('YYYY-MM')
    if (!monthMap.has(month)) {
      monthMap.set(month, 0)
    }
  }
  hexo.locals.get('posts').forEach(function (post) {
    const month = post.date.format('YYYY-MM')
    if (monthMap.has(month)) {
      monthMap.set(month, monthMap.get(month) + 1)
    }
  })

  const monthArr = JSON.stringify([...monthMap.keys()])
  const monthValueArr = JSON.stringify([...monthMap.values()])

  return `
  <script id="postsChart">
    let postsChart = echarts.init(document.getElementById('posts-chart'));
    let postsOption = {
        title: {
            show: false,
            text: ' ',
            top: -5,
            x: 'center'
        },
        tooltip: {
            trigger: 'axis'
        },
        xAxis: {
            type: 'category',
            data: ${monthArr}
        },
        yAxis: {
            type: 'value',
        },
        series: [
            {
                name: 'Number of Post',
                type: 'line',
                color: ['${color2}'],
                data: ${monthValueArr},
                markPoint: {
                    symbolSize: 40,
                    color: ['${color4}','${color3}','#82d3f4'],
                    data: [{
                        type: 'max',
                        itemStyle: {color: ['${color3}']},
                        name: 'MAX'
                    }, {
                        type: 'min',
                        itemStyle: {color: ['${color4}']},
                        name: 'MIN'
                    }]
                },
                markLine: {
                    itemStyle: {color: ['${color1}']},
                    data: [
                        {type: 'average', name: 'AVG'}
                    ]
                }
            }
        ]
    };
    postsChart.setOption(postsOption);
    </script>`
}

// Post Canlendar added by george16886
function postsCalendar () {
  // calculate range.
  const startDate = moment().subtract(0.75, 'years')
  const endDate = moment()
  const rangeArr = '["' + startDate.format('YYYY-MM-DD') + '", "' + endDate.format('YYYY-MM-DD') + '"]'

  // post and count map.
  const dateMap = new Map()
  hexo.locals.get('posts').forEach(function (post) {
    const date = post.date.format('YYYY-MM-DD')
    const count = dateMap.get(date)
    dateMap.set(date, count === null || count === undefined ? 1 : count + 1)
  })

  // loop the data for the current year, generating the number of post per day
  var i = 0
  var datePosts = '['
  const dayTime = 3600 * 24 * 1000
  for (let time = startDate; time <= endDate; time += dayTime) {
    const date = moment(time).format('YYYY-MM-DD')
    datePosts = (i === 0 ? datePosts + '["' : datePosts + ', ["') + date + '", ' + (dateMap.has(date) ? dateMap.get(date) : 0) + ']'
    i++
  }
  datePosts += ']'

  return `
  <script id="postsCalendar">
    let postsCalendar = echarts.init(document.getElementById('posts-calendar'));
    let postsCalendarOption = {
        title: {
            show: false,
            top: -3,
            text: ' ',
            left: 'center',
            textStyle: {
                color: '${color1}'
            }
        },
        tooltip: {
            padding: 5,
            backgroundColor: '#555',
            borderColor: '#777',
            borderWidth: 1,
            formatter: function (obj) {
                var value = obj.value;
                return '<div style="font-size: 14px;">' + value[0] + '：' + value[1] + '</div>';
            }
        },
        visualMap: {
            show: true,
            showLabel: true,
            categories: [0, 1, 2, 3, 4],
            calculable: true,
            inRange: {
                symbol: 'rect',
                color: ['#cdeef2', '#85e7f3', '#00d7f1 ', ' #00a7bb ', ' #007280 ']
            },
            itemWidth: 10,
            itemHeight: 10,
            orient: 'horizontal',
            left: 'center',
            bottom: 5
        },
        calendar: [{
            top: 'center',
            right: 'center',
            range: ${rangeArr},
            cellSize: [12, 12],
            splitLine: {
                show: false
            },
            itemStyle: {
                color: '${color3}',
                borderColor: '#fff',
                borderWidth: 2
            },
            yearLabel: {
                show: false,
                color: '#fff'
            },
            monthLabel: {
                nameMap: 'en',
                fontSize: 8,
                color: '#fff'
            },
            dayLabel: {
                formatter: '{start}  1st',
                nameMap: ['', 'M', '', 'W', '', 'F', ''],
                fontSize: 8,
                color: '#fff'
            }
        }],
        series: [{
            type: 'heatmap',
            coordinateSystem: 'calendar',
            calendarIndex: 0,
            data: ${datePosts}
        }]
    };
    postsCalendar.setOption(postsCalendarOption);
    </script>`
}

function tagsChart (dataLength = 10) {
  const tagArr = []
  hexo.locals.get('tags').map(function (tag) {
    tagArr.push({ name: tag.name, value: tag.length })
  })
  tagArr.sort((a, b) => { return b.value - a.value })

  const tagNameArr = []
  const tagCountArr = []
  for (let i = 0, len = Math.min(tagArr.length, dataLength); i < len; i++) {
    tagNameArr.push(tagArr[i].name)
    tagCountArr.push(tagArr[i].value)
  }

  const tagNameArrJson = JSON.stringify(tagNameArr)
  const tagCountArrJson = JSON.stringify(tagCountArr)

  return `
  <script id="tagsChart">
    let tagsChart = echarts.init(document.getElementById('tags-chart'));
    let tagsOption = {
        title: {
            show: false,
            text: ' ',
            top: -5,
            x: 'center'
        },
        tooltip: {},
        xAxis: [
            {
                type: 'category',
                data: ${tagNameArrJson}
            }
        ],
        yAxis: [
            {
                type: 'value'
            }
        ],
        series: [
            {
                type: 'bar',
                color: ['${color2}'],
                barWidth : 15,
                data: ${tagCountArrJson},
                markPoint: {
                    symbolSize: 40,
                    data: [{
                        type: 'max',
                        itemStyle: {color: ['${color3}']},
                        name: 'MAX'
                    }, {
                        type: 'min',
                        itemStyle: {color: ['${color4}']},
                        name: 'MIN'
                    }],
                },
                markLine: {
                    itemStyle: {color: ['${color1}']},
                    data: [
                        {type: 'average', name: 'AVG'}
                    ]
                }
            }
        ]
    };
    tagsChart.setOption(tagsOption);
    </script>`
}

function categoriesChart () { //only depth = 1
  const categoryArr = []
  hexo.locals.get('categories').map(function (category) {
    if (!category.parent) {
      categoryArr.push({ name: category.name, value: category.length })
  }})

  const categoryArrJson = JSON.stringify(categoryArr)

  return `
  <script id="categoriesChart">
    let categoriesChart = echarts.init(document.getElementById('categories-chart'));
    let categoriesOption = {
        title: {
            show: false,
            text: '',
            top: 0,
            x: 'center'
        },
        tooltip: {
            trigger: 'item',
            formatter: "{a} <br/>{b} : {c} ({d}%)"
        },
        series: [
            {
                name: 'Category:',
                type: 'pie',
                radius: '65%',
                color: ['#6772e5', '#ff9e0f', '${color4}', '${color3}', '#82d3f4', '#ab47bc', '#525f7f', '#f51c47', '#26A69A'],
                data: ${categoryArrJson},
                itemStyle: {
                    emphasis: {
                        shadowBlur: 10,
                        shadowOffsetX: 0,
                        shadowColor: 'rgba(0, 0, 0, 0.5)'
                    }
                }
            }
        ]
    };
    categoriesChart.setOption(categoriesOption);
    </script>`
}

// Category Radar added by george16886
function categoriesRadar (predefnames = "") {  // changing to tags by bf
  // const categories = hexo.locals.get('categories')
  var self_taglist = predefnames.split(',');
  const categories = []
 
  hexo.locals.get('tags').map(function (tag) {
    for (i=0; i<self_taglist.length; i++) {
      if (self_taglist[i]===tag.name){
        categories.push(tag)
  }}})

  // Find the maximum and average values of the post categories.
  const radarValueArr = [] // using square root function to scale the map better, radarValue is sqrted
  const realValueArr = [] 
  categories.some(function (category) {
    radarValueArr.push(Math.floor(Math.sqrt(category.length)))
    realValueArr.push(category.length)
  })
  const max = Math.max.apply(null, radarValueArr) + Math.min.apply(null, radarValueArr)

  // Calculate the data needed for the radar chart.
  const indicatorArr = []
  categories.map(function (category) {
    indicatorArr.push({ name: category.name, max: max })
  })

  const indicatorData = JSON.stringify(indicatorArr)
  const radarValueData = JSON.stringify(radarValueArr)

  return `
  <script id="categoriesRadar">
    let categoriesRadar = echarts.init(document.getElementById('categories-radar'));
    let categoriesRadarOption = {
        title: {
            show: false,
            left: 'center',
            text: '',
            textStyle: {
                fontWeight: 500,
                fontSize: 22
            }
        },
        tooltip: {},
        radar: {
            name: {
                textStyle: {
                    color: '${color1}'
                }
            },
            indicator: ${indicatorData},
            nameGap: 5,
            center: ['50%','55%'],
            radius: '75%'
        },
        series: [{
            type: 'radar',
            color: ['${color2}'],
            itemStyle: {normal: {areaStyle: {type: 'default'}}},
            data : [
                {
                    value : ${radarValueData},
                    name : 'Tags:'
                }
            ]
        }]
    };
    categoriesRadar.setOption(categoriesRadarOption);
    </script>`
}

==> hexo-s-c-dup1-1.0.1.tgz: package/index.js <==
// > 20200523
// This file is modified again by 5945
// changed categories chart to only depth 1 category
// changed categories radar to self defined tag radar

// This file is modified by george16886
// * Translated into English.
// * Remove title of category statistics chart.
// * Enlarge size of category statistics chart.
// * Add post canlendar.
// * Add category radar chart.

//color define
const color1 = '#FDEE34'; //neo yello
const color2 = '#00D7F1'; //neo blue
const color3 = '#3ecf8e'; //green
const color4 = '#fa755a'; //red

const cheerio = require('cheerio')
const moment = require('moment')

hexo.extend.filter.register('after_render:html', function (locals) {
  const $ = cheerio.load(locals)
  const post = $('#posts-chart')
  const tag = $('#tags-chart')
  const category = $('#categories-chart')
  const calendar = $('#posts-calendar')
  const radar = $('#categories-radar')
  let htmlEncode = false

  if (post.length > 0 || tag.length > 0 || category.length > 0 || calendar.length > 0 || radar.length > 0) {
    $('head').after('<style type="text/css">#posts-chart,#posts-calendar,#categories-chart,#categories-radar,#tags-chart{width: 100%;height: 300px;margin: 0.5rem auto;padding: 0.5rem;}</style><script type="text/javascript" src="https://cdn.jsdelivr.net/npm/echarts@4.7.0/dist/echarts.min.js"></script>')
    if (post.length > 0) {
      if (post.attr('data-encode') === 'true') htmlEncode = true
      post.after(postsChart())
    }
    if (calendar.length > 0) {
      if (calendar.attr('data-encode') === 'true') htmlEncode = true
      calendar.after(postsCalendar())
    }
    if (tag.length > 0) {
      if (tag.attr('data-encode') === 'true') htmlEncode = true
      tag.after(tagsChart(tag.attr('data-length')))
    }
    if (category.length > 0) {
      if (category.attr('data-encode') === 'true') htmlEncode = true
      category.after(categoriesChart())
    }
    if (radar.length > 0) {
      if (radar.attr('data-encode') === 'true') htmlEncode = true
      radar.after(categoriesRadar(radar.attr('predef-names')))
    }
    if (htmlEncode) {
      return $.root().html().replace(/&amp;#/g, '&#')
    } else {
      return $.root().html()
    }
  } else {
    return locals
  }
}, 25)

function postsChart () {
  const startDate = moment().subtract(2, 'years').startOf('month')
  const endDate = moment().endOf('month')

  const monthMap = new Map()
  const dayTime = 3600 * 24 * 1000
  for (let time = startDate; time <= endDate; time += dayTime) {
    const month = moment(time).format('YYYY-MM')
    if (!monthMap.has(month)) {
      monthMap.set(month, 0)
    }
  }
  hexo.locals.get('posts').forEach(function (post) {
    const month = post.date.format('YYYY-MM')
    if (monthMap.has(month)) {
      monthMap.set(month, monthMap.get(month) + 1)
    }
  })

  const monthArr = JSON.stringify([...monthMap.keys()])
  const monthValueArr = JSON.stringify([...monthMap.values()])

  return `
  <script id="postsChart">
    let postsChart = echarts.init(document.getElementById('posts-chart'));
    let postsOption = {
        title: {
            show: false,
            text: ' ',
            top: -5,
            x: 'center'
        },
        tooltip: {
            trigger: 'axis'
        },
        xAxis: {
            type: 'category',
            data: ${monthArr}
        },
        yAxis: {
            type: 'value',
        },
        series: [
            {
                name: 'Number of Post',
                type: 'line',
                color: ['${color2}'],
                data: ${monthValueArr},
                markPoint: {
                    symbolSize: 40,
                    color: ['${color4}','${color3}','#82d3f4'],
                    data: [{
                        type: 'max',
                        itemStyle: {color: ['${color3}']},
                        name: 'MAX'
                    }, {
                        type: 'min',
                        itemStyle: {color: ['${color4}']},
                        name: 'MIN'
                    }]
                },
                markLine: {
                    itemStyle: {color: ['${color1}']},
                    data: [
                        {type: 'average', name: 'AVG'}
                    ]
                }
            }
        ]
    };
    postsChart.setOption(postsOption);
    </script>`
}

// Post Canlendar added by george16886
function postsCalendar () {
  // calculate range.
  const startDate = moment().subtract(0.75, 'years')
  const endDate = moment()
  const rangeArr = '["' + startDate.format('YYYY-MM-DD') + '", "' + endDate.format('YYYY-MM-DD') + '"]'

  // post and count map.
  const dateMap = new Map()
  hexo.locals.get('posts').forEach(function (post) {
    const date = post.date.format('YYYY-MM-DD')
    const count = dateMap.get(date)
    dateMap.set(date, count === null || count === undefined ? 1 : count + 1)
  })

  // loop the data for the current year, generating the number of post per day
  var i = 0
  var datePosts = '['
  const dayTime = 3600 * 24 * 1000
  for (let time = startDate; time <= endDate; time += dayTime) {
    const date = moment(time).format('YYYY-MM-DD')
    datePosts = (i === 0 ? datePosts + '["' : datePosts + ', ["') + date + '", ' + (dateMap.has(date) ? dateMap.get(date) : 0) + ']'
    i++
  }
  datePosts += ']'

  return `
  <script id="postsCalendar">
    let postsCalendar = echarts.init(document.getElementById('posts-calendar'));
    let postsCalendarOption = {
        title: {
            show: false,
            top: -3,
            text: ' ',
            left: 'center',
            textStyle: {
                color: '${color1}'
            }
        },
        tooltip: {
            padding: 5,
            backgroundColor: '#555',
            borderColor: '#777',
            borderWidth: 1,
            formatter: function (obj) {
                var value = obj.value;
                return '<div style="font-size: 14px;">' + value[0] + '：' + value[1] + '</div>';
            }
        },
        visualMap: {
            show: true,
            showLabel: true,
            categories: [0, 1, 2, 3, 4],
            calculable: true,
            inRange: {
                symbol: 'rect',
                color: ['#e8f7f9', '#85e7f3', '#00d7f1 ', ' #00a7bb ', ' #007280 ']
            },
            itemWidth: 10,
            itemHeight: 10,
            orient: 'horizontal',
            left: 'center',
            bottom: 5
        },
        calendar: [{
            top: 'center',
            right: 'center',
            range: ${rangeArr},
            cellSize: [12, 12],
            splitLine: {
                show: false
            },
            itemStyle: {
                color: '${color3}',
                borderColor: '#fff',
                borderWidth: 2
            },
            yearLabel: {
                show: false,
                color: '#fff'
            },
            monthLabel: {
                nameMap: 'en',
                fontSize: 8,
                color: '#fff'
            },
            dayLabel: {
                formatter: '{start}  1st',
                nameMap: ['', 'M', '', 'W', '', 'F', ''],
                fontSize: 8,
                color: '#fff'
            }
        }],
        series: [{
            type: 'heatmap',
            coordinateSystem: 'calendar',
            calendarIndex: 0,
            data: ${datePosts}
        }]
    };
    postsCalendar.setOption(postsCalendarOption);
    </script>`
}

function tagsChart (dataLength = 10) {
  const tagArr = []
  hexo.locals.get('tags').map(function (tag) {
    tagArr.push({ name: tag.name, value: tag.length })
  })
  tagArr.sort((a, b) => { return b.value - a.value })

  const tagNameArr = []
  const tagCountArr = []
  for (let i = 0, len = Math.min(tagArr.length, dataLength); i < len; i++) {
    tagNameArr.push(tagArr[i].name)
    tagCountArr.push(tagArr[i].value)
  }

  const tagNameArrJson = JSON.stringify(tagNameArr)
  const tagCountArrJson = JSON.stringify(tagCountArr)

  return `
  <script id="tagsChart">
    let tagsChart = echarts.init(document.getElementById('tags-chart'));
    let tagsOption = {
        title: {
            show: false,
            text: ' ',
            top: -5,
            x: 'center'
        },
        tooltip: {},
        xAxis: [
            {
                type: 'category',
                data: ${tagNameArrJson}
            }
        ],
        yAxis: [
            {
                type: 'value'
            }
        ],
        series: [
            {
                type: 'bar',
                color: ['${color2}'],
                barWidth : 15,
                data: ${tagCountArrJson},
                markPoint: {
                    symbolSize: 40,
                    data: [{
                        type: 'max',
                        itemStyle: {color: ['${color3}']},
                        name: 'MAX'
                    }, {
                        type: 'min',
                        itemStyle: {color: ['${color4}']},
                        name: 'MIN'
                    }],
                },
                markLine: {
                    itemStyle: {color: ['${color1}']},
                    data: [
                        {type: 'average', name: 'AVG'}
                    ]
                }
            }
        ]
    };
    tagsChart.setOption(tagsOption);
    </script>`
}

function categoriesChart () { //only depth = 1
  const categoryArr = []
  hexo.locals.get('categories').map(function (category) {
    if (!category.parent) {
      categoryArr.push({ name: category.name, value: category.length })
  }})

  const categoryArrJson = JSON.stringify(categoryArr)

  return `
  <script id="categoriesChart">
    let categoriesChart = echarts.init(document.getElementById('categories-chart'));
    let categoriesOption = {
        title: {
            show: false,
            text: '',
            top: 0,
            x: 'center'
        },
        tooltip: {
            trigger: 'item',
            formatter: "{a} <br/>{b} : {c} ({d}%)"
        },
        series: [
            {
                name: 'Category:',
                type: 'pie',
                radius: '65%',
                color: ['#6772e5', '#ff9e0f', '${color4}', '${color3}', '#82d3f4', '#ab47bc', '#525f7f', '#f51c47', '#26A69A'],
                data: ${categoryArrJson},
                itemStyle: {
                    emphasis: {
                        shadowBlur: 10,
                        shadowOffsetX: 0,
                        shadowColor: 'rgba(0, 0, 0, 0.5)'
                    }
                }
            }
        ]
    };
    categoriesChart.setOption(categoriesOption);
    </script>`
}

// Category Radar added by george16886
function categoriesRadar (predefnames = "") {  // changing to tags by bf
  // const categories = hexo.locals.get('categories')
  var self_taglist = predefnames.split(',');
  const categories = []
 
  hexo.locals.get('tags').map(function (tag) {
    for (i=0; i<self_taglist.length; i++) {
      if (self_taglist[i]===tag.name){
        categories.push(tag)
  }}})

  // Find the maximum and average values of the post categories.
  const radarValueArr = [] // using square root function to scale the map better, radarValue is sqrted
  const realValueArr = [] 
  categories.some(function (category) {
    radarValueArr.push(Math.floor(Math.sqrt(category.length)))
    realValueArr.push(category.length)
  })
  const max = Math.max.apply(null, radarValueArr) + Math.min.apply(null, radarValueArr)

  // Calculate the data needed for the radar chart.
  const indicatorArr = []
  categories.map(function (category) {
    indicatorArr.push({ name: category.name, max: max })
  })

  const indicatorData = JSON.stringify(indicatorArr)
  const radarValueData = JSON.stringify(radarValueArr)

  return `
  <script id="categoriesRadar">
    let categoriesRadar = echarts.init(document.getElementById('categories-radar'));
    let categoriesRadarOption = {
        title: {
            show: false,
            left: 'center',
            text: '',
            textStyle: {
                fontWeight: 500,
                fontSize: 22
            }
        },
        tooltip: {},
        radar: {
            name: {
                textStyle: {
                    color: '${color1}'
                }
            },
            indicator: ${indicatorData},
            nameGap: 5,
            center: ['50%','55%'],
            radius: '75%'
        },
        series: [{
            type: 'radar',
            color: ['${color2}'],
            itemStyle: {normal: {areaStyle: {type: 'default'}}},
            data : [
                {
                    value : ${radarValueData},
                    name : 'Tags:'
                }
            ]
        }]
    };
    categoriesRadar.setOption(categoriesRadarOption);
    </script>`
}
